create table pv_power_measurement_t (
    time timestamp without time zone not null,
    deviceid text,
    status text,
    state integer,
    importEnergyActive double precision,
    importEnergyReactive double precision,
    exportEnergyActive double precision,
    exportEnergyReactive double precision,
    powerApparent double precision,
    powerActive double precision,
    powerReactive double precision,
    powerDemandPositive double precision,
    powerDemandReverse double precision,
    powerDemand double precision,
    factor double precision,
    angle double precision,
    voltage double precision,
    current double precision
);

select create_hypertable('pv_power_measurement_t', 'time');

grant insert on pv_power_measurement_t to nodered;
grant select on pv_power_measurement_t to grafana;

create view pv_stats_v as
  select time, importEnergyActive, importEnergyReactive, exportEnergyActive, exportEnergyReactive,
         powerApparent, powerActive, powerReactive, powerDemandPositive, powerDemandReverse, powerDemand,
         factor, angle, voltage, current
    from pv_power_measurement_t
    order by time;


create table pv_stats_t (
    id serial not null primary key,
    "date" date not null,
    dateType varchar(5) not null,
    first numeric(10,2) not null default 0,
    total numeric(10,2) not null default 0
);
alter table pv_stats_t add constraint ddT_uk unique ("date", dateType);

grant insert, select, update on pv_stats_t to nodered;
grant select, update on pv_stats_t_id_seq to nodered;

create or replace function pv_stats_func () 
    returns trigger
    language plpgsql 
as $$
declare
    v_dateTypes varchar[] := array['day', 'month', 'year'];
    v_dateType varchar;
begin
    foreach v_dateType in array v_dateTypes
    loop
      insert into pv_stats_t("date", dateType, first, total) 
        values (date_trunc(v_dateType, NEW.time::date), v_dateType, NEW.exportEnergyActive, 0)
        on conflict on constraint ddT_uk do update set total = NEW.exportEnergyActive - excluded.first;
    end loop;

    return NEW;
end;
$$

create trigger pv_stats_trig 
    after insert on pv_power_measurement_t
    for each row
    execute function pv_stats_func();



insert into pv_stats_t("date", dateType, first, total) values ('2023-12-12', 'day', 12, 0)
  on conflict on constraint ddT_uk do update set total = 15 - excluded.first;

;
